import 'dart:convert';

import 'dart:typed_data';
import 'package:encrypt/encrypt.dart';
import 'package:pointycastle/asymmetric/api.dart';
import "package:pointycastle/export.dart";

Key aesKey;
String robotPubKeyPem;
RSAPublicKey robotPubKey;

void generateAESKey() {
  aesKey = Key.fromSecureRandom(16);
}

Uint8List encryptPayload(Uint8List data) {
  final iv = IV.fromSecureRandom(16);
  final encrypter = Encrypter(AES(aesKey, mode: AESMode.cbc));
  final encrypted = encrypter.encryptBytes(data, iv: iv);
  return Uint8List.fromList([...iv.bytes, ...encrypted.bytes]);
}

Uint8List decryptPayload(Uint8List data) {
  final iv = data.getRange(0, 16);
  data = data.sublist(16);
  final encrypter = Encrypter(AES(aesKey, mode: AESMode.cbc));
  final decrypted = encrypter.decryptBytes(Encrypted(data), iv: IV(Uint8List.fromList(iv.toList())));
  return Uint8List.fromList(decrypted);
}

RSAPublicKey parsePublicKey(Uint8List pub) {
  final parser = RSAKeyParser();
  return parser.parse(utf8.decode(pub)) as RSAPublicKey;
}

Uint8List rsaEncrypt(RSAPublicKey myPublic, Uint8List dataToEncrypt) {
  final encryptor = PKCS1Encoding(RSAEngine())..init(true, PublicKeyParameter<RSAPublicKey>(myPublic)); // true=encrypt

  return _processInBlocks(encryptor, dataToEncrypt);
}

Uint8List rsaDecrypt(RSAPrivateKey myPrivate, Uint8List cipherText) {
  final decryptor = PKCS1Encoding(RSAEngine())
    ..init(false, PrivateKeyParameter<RSAPrivateKey>(myPrivate)); // false=decrypt

  return _processInBlocks(decryptor, cipherText);
}

Uint8List _processInBlocks(AsymmetricBlockCipher engine, Uint8List input) {
  final numBlocks = input.length ~/ engine.inputBlockSize + ((input.length % engine.inputBlockSize != 0) ? 1 : 0);

  final output = Uint8List(numBlocks * engine.outputBlockSize);

  var inputOffset = 0;
  var outputOffset = 0;
  while (inputOffset < input.length) {
    final chunkSize =
        (inputOffset + engine.inputBlockSize <= input.length) ? engine.inputBlockSize : input.length - inputOffset;

    outputOffset += engine.processBlock(input, inputOffset, chunkSize, output, outputOffset);

    inputOffset += chunkSize;
  }

  return (output.length == outputOffset) ? output : output.sublist(0, outputOffset);
}
